--
-- RandomlyMovingSpringPackages
--
--
-- @author John Deere 6930
-- @date 09/12/2016
--

RandomlyMovingSpringPackages = {};

function RandomlyMovingSpringPackages.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Cultivator, specializations) and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end

function RandomlyMovingSpringPackages:preLoad(savegame)	
end

function RandomlyMovingSpringPackages:load(savegame)	
	self.playAnimationBackwards = RandomlyMovingSpringPackages.playAnimationBackwards;
	self.getSpringGroups = RandomlyMovingSpringPackages.getSpringGroups;
	self.startSpringAnimation = RandomlyMovingSpringPackages.startSpringAnimation;
	self.setTimeOffset = RandomlyMovingSpringPackages.setTimeOffset;
	self.getRandomSpringPacks = RandomlyMovingSpringPackages.getRandomSpringPacks;
	self.getIsSpringPackActive = RandomlyMovingSpringPackages.getIsSpringPackActive;
	self.setRandomAnimStopTime = RandomlyMovingSpringPackages.setRandomAnimStopTime;
	
	
	self.activePlayingSpringPacks = {};
	
	self.springPacks = {};	
	local i=0;
	while true do
		local baseName = string.format("vehicle.springPackagesRandom.springPack(%d)", i);
		if not hasXMLProperty(self.xmlFile, baseName) then
			break;
		end;
		local animationName = getXMLString(self.xmlFile, baseName.. "#animationName");
		local springAnimationSpeedMax = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.. "#springAnimationSpeedMax"), 1);
		local springAnimationSpeedMin = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.. "#springAnimationSpeedMin"), 1);
		if animationName ~= nil then
			if self.animations[animationName] ~= nil then
				local entry = {};
				entry.isActive = false;
				entry.springAnimation = animationName;
				entry.springAnimationSpeedMax = springAnimationSpeedMax;
				entry.springAnimationSpeedMin = springAnimationSpeedMin;
				
				entry.springTimeOut = 0;
				entry.timeOffsetMax = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.. "#timeOffsetMax"), 1) * 1000;
				entry.timeOffsetMin = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.. "#timeOffsetMin"), 1) * 1000;
				entry.lastOffset = 0;
							
				table.insert(self.springPacks, entry);
			end;
		end;
		i = i + 1;
	end;	
	
	self.springGroups = {};
	local i=0;
	while true do
		local baseName = string.format("vehicle.springPackagesRandom.springGroups(%d)", i);
		if not hasXMLProperty(self.xmlFile, baseName) then
			break;
		end;
		local index = getXMLInt(self.xmlFile, baseName.. "#index");
		local indices = Utils.getVectorNFromString(getXMLString(self.xmlFile, baseName.. "#childIndicies"));
		if index ~= nil and indices ~= nil then
			local entry = {};
			entry.parentPart = self.springPacks[index+1];			
			entry.childs = {};			
			for _,childIndex in pairs(indices) do
				local childPart = self.springPacks[childIndex+1];
				childPart.isChild = true;
				table.insert(entry.childs, childPart);
			end;			
			table.insert(self.springGroups, entry);
		end;
		i = i + 1;
	end;
end

function RandomlyMovingSpringPackages:postLoad(savegame)
	for _,springPack in pairs(self.springPacks) do
		self:setTimeOffset(springPack);
	end;
end

function RandomlyMovingSpringPackages:delete()
end

function RandomlyMovingSpringPackages:mouseEvent(posX, posY, isDown, isUp, button)
end

function RandomlyMovingSpringPackages:keyEvent(unicode, sym, modifier, isDown)
end

function RandomlyMovingSpringPackages:update(dt)
	if self:getIsActive() then
		if self.doGroundManipulation then
			if self:getLastSpeed() > 3 then
				local springPack = self:getRandomSpringPacks();
				if springPack ~= nil then
					if self:getIsSpringPackActive(springPack, false) then
						self:startSpringAnimation(springPack);
					end;
					
					for _,springPack in pairs(self.activePlayingSpringPacks) do
						if not self:getIsAnimationPlaying(springPack.springAnimation) then
							self:playAnimationBackwards(springPack, false);
						end;
					end;
				end;
			end;
		else
			for _,springPack in pairs(self.activePlayingSpringPacks) do
				if self:getAnimationTime(springPack.springAnimation) > 0 then
					self:stopAnimation(springPack.springAnimation, false);
					self:playAnimationBackwards(springPack, true);
				end;
			end;
		end;
	end;
end;

function RandomlyMovingSpringPackages:updateTick(dt)
end;

function RandomlyMovingSpringPackages:draw()
end;

function RandomlyMovingSpringPackages:playAnimationBackwards(springPack, maxSpeed)
	local speed = 1;
	if maxSpeed then
		speed = springPack.springAnimationSpeedMax;
	else
		speed = self.animations[springPack.springAnimation].currentSpeed;
	end;
	self:playAnimation(springPack.springAnimation, -speed, nil, false);
	self.activePlayingSpringPacks[springPack] = nil;
end;

function RandomlyMovingSpringPackages:getSpringGroups(springPack)
	for _,group in pairs(self.springGroups) do
		if group.parentPart == springPack then
			return group.childs;
		else
			return nil;
		end;
	end;
end;

function RandomlyMovingSpringPackages:startSpringAnimation(springPack)
	self:setRandomAnimStopTime(springPack);
	local currentSpeed = self.animations[springPack.springAnimation].currentSpeed;
	local speed = Utils.clamp(currentSpeed, springPack.springAnimationSpeedMin, springPack.springAnimationSpeedMax);
	self:playAnimation(springPack.springAnimation, speed, nil, false);
	local springGroups = self:getSpringGroups(springPack)
	if springGroups ~= nil then
		for k,part in pairs(springGroups) do
			if self:getIsSpringPackActive(part, true) then
				self:startSpringAnimation(part);
			end;
		end;
	end;
	self:setTimeOffset(springPack);
	self.activePlayingSpringPacks[springPack] = springPack;
end;

function RandomlyMovingSpringPackages:setTimeOffset(springPack)
	local percent = math.random(0, 100);
	springPack.lastOffset = springPack.timeOffsetMin + springPack.timeOffsetMax*(percent/100);
	springPack.springTimeOut = g_currentMission.time+springPack.lastOffset;
end;

function RandomlyMovingSpringPackages:getRandomSpringPacks()
	local retVal = nil;
	local num = table.getn(self.springPacks);
	local i = math.random(1, num);
	if i <= num then
		retVal = self.springPacks[i];
	end;
	return retVal;
end;

function RandomlyMovingSpringPackages:getIsSpringPackActive(springPack, forced)
	local retVal = false;
	if springPack.springTimeOut <= g_currentMission.time then
		retVal = true;
	end;
	if forced then
		retVal = true;
	end;
	if self:getIsAnimationPlaying(springPack.springAnimation) then
		retVal = false;
	end;
	return retVal;
end;

function RandomlyMovingSpringPackages:setRandomAnimStopTime(springPack)
	local percent = math.random(0, 100);
	local stopTime = percent/100;
	self:setAnimationStopTime(springPack.springAnimation, stopTime)
end;